{----------------------------------------------------------------------------
|
| Module: WizForm.Pas
|
| Description: Main form for ClassGen wizard.
|              references.
|
| History:       , 1998. MB first version.
|          Oct 16, 2001. MB Changed for Spider ODB version 2
|
|---------------------------------------------------------------------------}
unit WizForm;

interface

uses
    OdbIndex, { for TOdbIndex }
    OdbField, { for TOdbField }

  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Menus;

type
  TWizardForm = class(TForm)
    NoteBook: TNoteBook;
    BottomPanel: TPanel;
    CancelBtn: TBitBtn;
    BackBtn: TBitBtn;
    NextBtn: TBitBtn;
    FinishBtn: TBitBtn;
    Bevel: TBevel;
    ImagePanel: TPanel;
    Image: TImage;
    IntroLabel: TLabel;
    GeneralLabel: TLabel;
    FileNameLbl: TLabel;
    FileNameEdit: TEdit;
    FileNameBrowseBtn: TBitBtn;
    ClassNameEdit: TEdit;
    ClassNameLbl: TLabel;
    OpenDialog: TOpenDialog;
    DataMemberLB: TListBox;
    DataMembersLbl: TLabel;
    AddDataMemberBtn: TBitBtn;
    DeleteDataMemberBtn: TBitBtn;
    EditDataMemberBtn: TBitBtn;
    DataMemberPopup: TPopupMenu;
    Add1: TMenuItem;
    Edit1: TMenuItem;
    Delete1: TMenuItem;
    AncestorClassEdit: TEdit;
    AncestorClassLbl: TLabel;
    FieldsAndIndicesLbl: TLabel;
    FieldLB: TListBox;
    FieldsBevel: TBevel;
    AddFieldBtn: TBitBtn;
    EditFieldBtn: TBitBtn;
    DeleteFieldBtn: TBitBtn;
    FieldsLbl: TLabel;
    IndicesBevel: TBevel;
    IndicesLbl: TLabel;
    IndexLB: TListBox;
    AddIndexBtn: TBitBtn;
    EditIndexBtn: TBitBtn;
    DeleteIndexBtn: TBitBtn;
    FieldPopup: TPopupMenu;
    Add2: TMenuItem;
    Edit2: TMenuItem;
    Delete2: TMenuItem;
    IndexPopup: TPopupMenu;
    Add3: TMenuItem;
    Edit3: TMenuItem;
    Delete3: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure MoveBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FileNameBrowseBtnClick(Sender: TObject);
    procedure FinishBtnClick(Sender: TObject);
    procedure AddDataMemberBtnClick(Sender: TObject);
    procedure EditDataMemberBtnClick(Sender: TObject);
    procedure DeleteDataMemberBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AddFieldBtnClick(Sender: TObject);
    procedure EditFieldBtnClick(Sender: TObject);
    procedure DeleteFieldBtnClick(Sender: TObject);
    procedure AddIndexBtnClick(Sender: TObject);
    procedure EditIndexBtnClick(Sender: TObject);
    procedure DeleteIndexBtnClick(Sender: TObject);
  private
    { Private declarations }
    procedure AdjustButtons;
    procedure BeforePageChange( const CurrentPage  : LongInt;
                                const NewPage      : LongInt;
                                var   AcceptChange : Boolean );


   procedure AddFieldNamesToStrings( const Strings : TStrings );

   function FieldUsedByAnyIndex( const FieldName : String ) : Boolean;

   function FieldUsedByIndex( const FieldName : String;
                              const Index     : TOdbIndex ) : Boolean;

  public
    { Public declarations }

   { return the TOdbField corresonding to an index in the field
     list box. }
   function GetField( const Index : Integer ) : TOdbField;

   { return the TOdbIndex corresonding to an index in the index
     list box. }
   function GetIndex( const Index : Integer ) : TOdbIndex;

   function MemberName( const Index : Integer ) : String;
   function MemberTypeStr( const Index : Integer ) : String;

   { returns a dmtXXXX constant given a DataMemberLB index. }
   function MemberType( const Index : Integer ) : Word;

   function ClassNameWithoutT : String;

   function ORMappingName : String;

   { Returns the autoincrment field or nil if there is not any auto inc field. }
   function GetAutoIncField : TOdbField;

  end;

var
  WizardForm: TWizardForm;

const

{ constant for data member types. }
dmtBoolean    =  0;
dmtByte       =  1;
dmtChar       =  2;
dmtComp       =  3;
dmtDouble     =  4;
dmtExtended   =  5;
dmtInteger    =  6;
dmtLongInt    =  7;
dmtShortInt   =  8;
dmtSingle     =  9;
dmtString     = 10;
dmtWord       = 11;
dmtObject     = 12;

dmtMin        =  0;
dmtMax        = 12;

function DataMemberFieldTypeStr( const FieldType : Word ) : String;
       
{--------------------------------------------------------------------------}

implementation

{$R *.DFM}

uses
    ObjBuckt, { for TStringCombo }
    CodeGen,  { for CreateUnit }
    AddMemFm, { for AddDataMemberForm }
    AddFldFm, { for AddFieldForm }
    AddIdxFm, { for AddIndexForm }
    OdbMisc;  { for RightPad }

{--------------------------------------------------------------------------}

function DataMemberFieldTypeStr( const FieldType : Word ) : String;
begin
    case FieldType of
        dmtBoolean    : Result := 'Boolean';
        dmtByte       : Result := 'Byte';
        dmtChar       : Result := 'Char';
        dmtComp       : Result := 'Comp';
        dmtDouble     : Result := 'Double';
        dmtExtended   : Result := 'Extended';
        dmtInteger    : Result := 'Integer';
        dmtLongInt    : Result := 'LongInt';
        dmtShortInt   : Result := 'ShortInt';
        dmtSingle     : Result := 'Single';
        dmtString     : Result := 'String';
        dmtWord       : Result := 'Word';
        dmtObject     : Result := 'Object';
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.FormShow(Sender: TObject);
begin
  NoteBook.PageIndex := 0;
  AdjustButtons;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.CancelBtnClick(Sender: TObject);
begin
    if (MessageDlg( 'Are you sure you wish to cancel ?',
                   mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
        Close;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.MoveBtnClick(Sender: TObject);
var
    AcceptChange : Boolean;
begin

    AcceptChange := True;

    if Sender=NextBtn then
    begin
        BeforePageChange( Notebook.PageIndex,
                          Notebook.PageIndex + 1,
                          AcceptChange );
        if not AcceptChange then
            Exit;

        NoteBook.PageIndex:=NoteBook.PageIndex+1;
    end
    else if Sender=BackBtn then
    begin
        BeforePageChange( Notebook.PageIndex,
                          Notebook.PageIndex - 1,
                          AcceptChange );
        if not AcceptChange then
            Exit;

        NoteBook.PageIndex:=NoteBook.PageIndex-1;
    end;

    AdjustButtons;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.AdjustButtons;
begin
    BackBtn.Visible   := (Notebook.PageIndex >= 1);
    NextBtn.Visible   := (Notebook.PageIndex < (Notebook.Pages.Count-1));
    FinishBtn.Visible := (Notebook.PageIndex = (Notebook.Pages.Count-1));
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.FormCreate(Sender: TObject);
begin
    IntroLabel.Caption := 'Welcome to the Spider Object Database class generator ' +
                          'wizard. This wizard will create a unit containing a persistent ' +
                          'object and the related TORMapping descendant class.';

    { ensure the list boxes have horizontal scroll bars to
      accomadate different screen resolutions.}
    SendMessage(DataMemberLB.Handle, LB_SetHorizontalExtent, 500, LongInt(0));
    SendMessage(IndexLB.Handle, LB_SetHorizontalExtent, 500, LongInt(0));
    SendMessage(FieldLB.Handle, LB_SetHorizontalExtent, 500, LongInt(0));
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.FormDestroy(Sender: TObject);
var
    Index : Integer;
begin
    for Index := 0 to FieldLB.Items.Count-1 do
        FieldLB.Items.Objects[Index].Free;

    for Index := 0 to IndexLB.Items.Count-1 do
        IndexLB.Items.Objects[Index].Free;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.FileNameBrowseBtnClick(Sender: TObject);
begin
    OpenDialog.FileName := FileNameEdit.Text;
    if OpenDialog.Execute then
        FileNameEdit.Text := OpenDialog.FileName;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.BeforePageChange( const CurrentPage  : LongInt;
                            const NewPage      : LongInt;
                            var   AcceptChange : Boolean );
begin
    AcceptChange := True;

    case CurrentPage of
        0 :
        begin
            AcceptChange := (FileNameEdit.Text <> '')
                            and
                            (ClassNameEdit.Text <> '');
            if not AcceptChange then
                MessageDlg( 'Value is required.', mtError, [mbOk], 0);
        end;
        1 :
        begin
            if NewPage = 2 then
            begin
                AcceptChange := (DataMemberLB.Items.Count > 0);
                if not AcceptChange then
                    MessageDlg( 'The data member list is empty.',
                                mtError, [mbOk], 0);
            end;
        end;
    end;
end;

{--------------------------------------------------------------------------}

function TWizardForm.MemberName( const Index : Integer ) : String;
begin
    Result := Trim(ExtractWord(1, DataMemberLB.Items[Index], [':']));
end;

{--------------------------------------------------------------------------}

function TWizardForm.MemberType( const Index : Integer ) : Word;
begin
    Result := LongInt(DataMemberLB.Items.Objects[Index]);
end;

{--------------------------------------------------------------------------}

function TWizardForm.MemberTypeStr( const Index : Integer ) : String;
var
    DataType : Word;
begin
    DataType := MemberType(Index);

    if DataType <> dmtObject then
        Result := DataMemberFieldTypeStr(DataType)
    else
        Result := Trim(ExtractWord(2, DataMemberLB.Items[Index], [':']));
end;

{--------------------------------------------------------------------------}

function TWizardForm.ClassNameWithoutT : String;
begin
    Result := ClassNameEdit.Text;
    Delete(Result, 1, 1);
end;

{--------------------------------------------------------------------------}

function TWizardForm.ORMappingName : String;
begin
    Result := ClassNameEdit.Text + 'ORMapping';
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.FinishBtnClick(Sender: TObject);
begin
    if CreateUnit( Self, FileNameEdit.Text ) then
        Self.Close;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.AddDataMemberBtnClick(Sender: TObject);
var
    Form        : TAddDataMemberForm;
    DataType    : Word;
    DataTypeStr : String;
begin
    Form := TAddDataMemberForm.Create(Self);
    try
        Form.Caption := 'Add Data Member';
        if Form.ShowModal = mrOk then
        begin
            DataType := LongInt(Form.MemberTypeCB.ItemIndex);

            if DataType = dmtObject then
                DataTypeStr := Form.MemberClassNameEdit.Text
            else
                DataTypeStr := Form.MemberTypeCB.Text;

            DataMemberLB.Items.AddObject(
                              RightPad( Form.MemberNameEdit.Text, ' ', 20) +
                              ' : ' + DataTypeStr, TObject(DataType));
        end;
    finally
        Form.Free;
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.EditDataMemberBtnClick(Sender: TObject);
var
    Form        : TAddDataMemberForm;
    Index       : Integer;
    DataType    : Word;
    DataTypeStr : String;
begin
    Index := DataMemberLB.ItemIndex;
    if Index = -1 then
        Exit;
    Form := TAddDataMemberForm.Create(Self);
    try
        Form.Caption := 'Edit Data Member';
        Form.MemberNameEdit.Text    := MemberName(Index);
        Form.MemberTypeCB.ItemIndex := LongInt(DataMemberLB.Items.Objects[Index]);

        if Form.ShowModal = mrOk then
        begin
            DataType := LongInt(Form.MemberTypeCB.ItemIndex);

            if DataType = dmtObject then
                DataTypeStr := Form.MemberClassNameEdit.Text
            else
                DataTypeStr := Form.MemberTypeCB.Text;

            DataMemberLB.Items[Index] := RightPad( Form.MemberNameEdit.Text, ' ', 20) +
                                         ' : ' + DataTypeStr;

            DataMemberLB.Items.Objects[Index] := TObject(DataType);
        end;
    finally
        Form.Free;
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.DeleteDataMemberBtnClick(Sender: TObject);
var
    Index : LongInt;
begin
    if MessageDlg( 'Delete selected data members ?',
                   mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
            Exit;

    Index := 0;
    while Index < DataMemberLB.Items.Count do
    begin
        if DataMemberLB.Selected[Index] then
            DataMemberLB.Items.Delete(Index)
        else
            Inc(Index);
    end;
end;

{--------------------------------------------------------------------------}

function TWizardForm.GetField( const Index : Integer ) : TOdbField;
begin
    Result := TOdbField(FieldLB.Items.Objects[Index]);
end;

{--------------------------------------------------------------------------}

function TWizardForm.GetAutoIncField : TOdbField;
var
    Index  : Integer;
begin
    Result := nil;
    for Index := 0 to FieldLB.Items.Count-1 do
    begin
         Result := GetField(Index);

         if Result.FieldType = oftAutoInc then
             Exit
         else
             Result := nil;
    end;
end;

{--------------------------------------------------------------------------}

function TWizardForm.GetIndex( const Index : Integer ) : TOdbIndex;
begin
    Result := TOdbIndex(IndexLB.Items.Objects[Index]);
end;

{--------------------------------------------------------------------------}

function TWizardForm.FieldUsedByIndex( const FieldName : String;
                                       const Index     : TOdbIndex ) : Boolean;
var
    StringCombo : TStringCombo;
begin
    StringCombo := TStringCombo.Create(FieldName, nil);
    try
        Result := Index.FieldNames.Search(StringCombo, 1);
    finally
        StringCombo.Free;
    end;
end;

{--------------------------------------------------------------------------}

function TWizardForm.FieldUsedByAnyIndex( const FieldName : String ) : Boolean;
var
    Index       : Integer;
    OdbIndex    : TOdbIndex;
begin
    Result := False;

    for Index := 0 to IndexLB.Items.Count-1 do
    begin
        OdbIndex := GetIndex(Index);

        if FieldUsedByIndex(FieldName, OdbIndex) then
        begin
            Result := True;
            Exit;
        end;
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.AddFieldBtnClick(Sender: TObject);
var
    Form       : TAddFieldForm;
    OdbField   : TOdbField;
    StringSize : Integer;
    FieldType  : TOdbFieldType;
    Desc       : String;
begin
    Form := TAddFieldForm.Create(Self);
    try
        Form.Caption := 'Add Field';
        if Form.ShowModal = mrOk then
        begin
            FieldType := TOdbFieldType(Form.FieldTypeCB.ItemIndex);

            if (FieldType = oftAutoInc) and
               (GetAutoIncField <> nil) then
            begin
                MessageDlg( 'Only one autoincrement field may be defined.', mtError, [mbOk], 0);
                Exit;
            end;

            Desc := RightPad( Form.FieldNameEdit.Text, ' ', 20) +
                              ' : ' +
                              Form.FieldTypeCB.Items[Form.FieldTypeCB.ItemIndex];

            StringSize := 0;
            if FieldType = oftString then
            begin
                StringSize := StrToInt(Form.StringSizeEdit.Text);
                Desc       := Desc + '[' + IntToStr(StringSize-1) + ']';
            end;

            OdbField := TOdbField.Create( Form.FieldNameEdit.Text, FieldType, StringSize );

            FieldLB.Items.AddObject(Desc, OdbField);
        end;
    finally
        Form.Free;
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.EditFieldBtnClick(Sender: TObject);
var
    Form             : TAddFieldForm;
    OdbField         : TOdbField;
    Desc             : String;
    CurrentIsAutoInc : Boolean;
begin
    if FieldLB.ItemIndex = -1 then
        Exit;

    Form := TAddFieldForm.Create(Self);
    try
        Form.Caption := 'Edit Field';
        OdbField     := GetField(FieldLB.ItemIndex);
        Form.FieldTypeCB.ItemIndex := Ord(OdbField.FieldType);
        Form.FieldNameEdit.Text    := OdbField.Name;
        Form.StringSizeEdit.Text   := IntToStr(OdbField.Size);

        CurrentIsAutoInc           := (OdbField.FieldType = oftAutoInc);

        if Form.ShowModal = mrOk then
        begin
            if not CurrentIsAutoInc and
               (GetAutoIncField <> nil) and
               (TOdbFieldType(Form.FieldTypeCB.ItemIndex) = oftAutoInc) then
            begin
                MessageDlg( 'Only one autoincrement field may be defined.', mtError, [mbOk], 0);
                Exit;
            end;

            OdbField.FieldType := TOdbFieldType(Form.FieldTypeCB.ItemIndex);
            OdbField.Name      := Form.FieldNameEdit.Text;
            OdbField.Size      := 0;
            if OdbField.FieldType = oftString then
                OdbField.Size := StrToInt(Form.StringSizeEdit.Text);

            Desc := RightPad( OdbField.Name, ' ', 20) +
                              ' : ' +
                              Form.FieldTypeCB.Items[Ord(OdbField.FieldType)];

            if OdbField.Size > 0 then
                Desc := Desc + '[' + IntToStr(OdbField.Size-1) + ']';

            FieldLB.Items[FieldLB.ItemIndex] := Desc;
        end;
    finally
        Form.Free;
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.DeleteFieldBtnClick(Sender: TObject);
var
    Index : LongInt;
    Field : TOdbField;
begin
    if MessageDlg( 'Delete selected fields ?',
                   mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
            Exit;

    Index := 0;
    while Index < FieldLB.Items.Count do
    begin
        if FieldLB.Selected[Index] then
        begin
            Field := GetField(Index);

            if FieldUsedByAnyIndex( Field.Name ) then
            begin
                MessageDlg( 'Unable to delete field ' + Field.Name + '.'#13 +
                            'Field is used by an index.',
                            mtInformation, [mbOk], 0);
                Inc(Index);
            end
            else
            begin
                Field.Free;
                FieldLB.Items.Delete(Index);
            end;
        end
        else
            Inc(Index);
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.AddFieldNamesToStrings( const Strings : TStrings );
var
    Index : Integer;
begin
    Strings.Clear;

    for Index := 0 to FieldLB.Items.Count-1 do
        Strings.Add( GetField(Index).Name );
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.AddIndexBtnClick(Sender: TObject);
var
    Form        : TAddIndexForm;
    OdbIndex    : TOdbIndex;
    Index       : Integer;
    Attributes  : TOdbIndexAttributes;
begin
    Form := TAddIndexForm.Create(Self);
    try
        Form.Caption := 'Add Index';
        AddFieldNamesToStrings(Form.FieldsLb.Items);
        Form.PrimaryCB.Checked := (IndexLB.Items.Count=0);

        if Form.ShowModal = mrOk then
        begin
            Attributes := [];
            if Form.PrimaryCB.Checked then
                Attributes := Attributes + [iaPrimary];
            if Form.UniqueCB.Checked then
                Attributes := Attributes + [iaUnique];
            if Form.DescendingCB.Checked then
                Attributes := Attributes + [iaDescending];
            if Form.CaseInsensitiveCB.Checked then
                Attributes := Attributes + [iaCaseInsensitive];

            OdbIndex := TOdbIndex.Create( Form.IndexNameEdit.Text, Attributes );

            for Index := 0 to Form.IndexFieldsLB.Items.Count-1 do
                OdbIndex.FieldNames.Insert(TStringCombo.Create(Form.IndexFieldsLB.Items[Index], nil));

            IndexLB.Items.AddObject(OdbIndex.Name, OdbIndex);
        end;
    finally
        Form.Free;
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.EditIndexBtnClick(Sender: TObject);
var
    Form        : TAddIndexForm;
    OdbIndex    : TOdbIndex;
    Attributes  : TOdbIndexAttributes;
    StringCombo : TStringCombo;
begin
    if IndexLB.ItemIndex = -1 then
        Exit;

    Form := TAddIndexForm.Create(Self);
    try
        Form.Caption := 'Edit Index';
        AddFieldNamesToStrings(Form.FieldsLb.Items);
        OdbIndex := GetIndex(IndexLB.ItemIndex);

        Form.IndexNameEdit.Text := OdbIndex.Name;

        Form.PrimaryCB.Checked         := (iaPrimary in OdbIndex.Attributes);
        Form.UniqueCB.Checked          := (iaUnique in OdbIndex.Attributes);
        Form.DescendingCB.Checked      := (iaDescending in OdbIndex.Attributes);
        Form.CaseInsensitiveCB.Checked := (iaCaseInsensitive in OdbIndex.Attributes);

        if OdbIndex.FieldNames.GotoFirst then
        repeat
            StringCombo := OdbIndex.FieldNames.CurrentObj as TStringCombo;

            Form.IndexFieldsLB.Items.Add(StringCombo.Str);
            Form.FieldsLB.Items.Delete(Form.FieldsLB.Items.IndexOf(StringCombo.Str));
        until not OdbIndex.FieldNames.GotoNext;

        if Form.ShowModal = mrOk then
        begin
            Attributes := [];
            if Form.PrimaryCB.Checked then
                Attributes := Attributes + [iaPrimary];
            if Form.UniqueCB.Checked then
                Attributes := Attributes + [iaUnique];
            if Form.DescendingCB.Checked then
                Attributes := Attributes + [iaDescending];
            if Form.CaseInsensitiveCB.Checked then
                Attributes := Attributes + [iaCaseInsensitive];

            OdbIndex.Name       := Form.IndexNameEdit.Text;
            OdbIndex.Attributes := Attributes;

            IndexLB.Items[IndexLB.ItemIndex] := OdbIndex.Name;
        end;
    finally
        Form.Free;
    end;
end;

{--------------------------------------------------------------------------}

procedure TWizardForm.DeleteIndexBtnClick(Sender: TObject);
var
    Index    : LongInt;
begin
    if MessageDlg( 'Delete selected indices ?',
                   mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
            Exit;

    Index := 0;
    while Index < IndexLB.Items.Count do
    begin
        if IndexLB.Selected[Index] then
        begin
            GetIndex(Index).Free;
            IndexLB.Items.Delete(Index);
        end
        else
            Inc(Index);

    end;
end;

{--------------------------------------------------------------------------}

end.
