{---------------------------------------------------------------------------
|
|  Library: Spider Object Database
|
|  Module: OdbDialg.Pas
|
|  Description: Spider object database dialog boxes.
|
|  History: May 23, 1998. MB first version.
|
|---------------------------------------------------------------------------}
unit OdbDialg;

{$I Odb.Inc}

interface

uses
    Classes,  { for TComponent }
    Controls, { for TWinControl }
    StdCtrls, { for TComboBox, TEdit }
    ObjList,  { for TUnOrderedList }
    Odb;      { for TObjectDatabase }

type

TOpenDatabaseOption = ( ShowDriver,
                        ShowDatabase,
                        ShowServer,
                        ShowUser );

TOpenDatabaseOptions = set of TOpenDatabaseOption;

TOpenDatabaseDialog = class(TComponent)
protected
    FDriverList      : TStrings;
    FHistoryFile     : String;
    FHistorySize     : Word;
    FOptions         : TOpenDatabaseOptions;
    FDefaultDriver   : String;
    FDefaultDatabase : String;
    FDefaultServer   : String;
    FDefaultUser     : String;

    FFullHistoryFile : String;

    FLastDriver      : String;
    FDatabaseHistory : TUnOrderedList;
    FServerHistory   : TUnOrderedList;
    FUserHistory     : TUnOrderedList;

    procedure SetDriverList( const InDriverList : TStrings );

    procedure Loaded; override;

public
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;

    { executes the dialog box. On success, a opened database is returned.
      The user must later free this object. On user cancel, nil is
      returned. }
    function Execute : TObjectDatabase;

    property LastDriver       : String read FLastDriver;         
    property DatabaseHistory  : TUnOrderedList read FDatabaseHistory;
    property ServerHistory    : TUnOrderedList read FServerHistory;
    property UserHistory      : TUnOrderedList read FUserHistory;

published

    property DriverList : TStrings read FDriverList write SetDriverList;

    property HistoryFile : String read FHistoryFile write FHistoryFile;

    property HistorySize : Word read FHistorySize write FHistorySize
                                 default 8;

    property Options : TOpenDatabaseOptions read FOptions write FOptions
                                            default [ShowDriver, ShowDatabase];

    property DefaultDriver   : String read FDefaultDriver write FDefaultDriver;
    property DefaultDatabase : String read FDefaultDatabase write FDefaultDatabase;
    property DefaultServer   : String read FDefaultServer write FDefaultServer;
    property DefaultUser     : String read FDefaultUser write FDefaultUser;

end;

{--------------------------------------------------------------------}

implementation

uses
    Forms,    { for TForm }
    Buttons,  { for TBitBtn }
    SysUtils, { for FileExists } 
    ObjBuckt, { for TStringCombo }
    ObjPerst, { for TObjectStream }
    ObjCmp;   { for CompareStringCombo }

type

TLoginDialog = class(TForm)
protected
    OpenDatabaseDialog : TOpenDatabaseDialog;
    DriverCB           : TComboBox;
    DatabaseCB         : TComboBox;
    ServerCB           : TComboBox;
    UserCB             : TComboBox;
    PasswordEdit       : TEdit;
public
    constructor CreateStatic(
                      const InOpenDatabaseDialog : TOpenDatabaseDialog );
end;

constructor TLoginDialog.CreateStatic(
                       const InOpenDatabaseDialog : TOpenDatabaseDialog );
const
    CFieldPos    = 120;
    CLabelPos    = 8;
    CFieldHeight = 40;
    CFieldWidth  = 244;
var
    VPos        : Integer;
    DriverNo    : Integer;
    StringCombo : TStringCombo;
    ItemIndex   : Integer;
begin
    inherited CreateNew(nil);

    OpenDatabaseDialog := InOpenDatabaseDialog;

    Font.Name   := 'Arial';
    Font.Size   := 10;
    Position    := poScreenCenter;
    BorderStyle := bsDialog;
    Width       := 380;
    Caption     := 'Open Database';

    DriverCB     := TComboBox.Create(Self);
    DatabaseCB   := TComboBox.Create(Self);
    ServerCB     := TComboBox.Create(Self);
    UserCB       := TComboBox.Create(Self);
    PasswordEdit := TEdit.Create(Self);

    DriverCB.Visible     := False;
    DatabaseCB.Visible := False;
    ServerCB.Visible   := False;
    UserCB.Visible     := False;
    PasswordEdit.Visible := False;

    VPos := 8;
    with OpenDatabaseDialog do
    begin
        DriverCB.Text      := DefaultDriver;
        DatabaseCB.Text    := DefaultDatabase;
        ServerCB.Text      := DefaultServer;
        UserCB.Text        := DefaultUser;
        PasswordEdit.Text  := '';

        if ShowDriver in Options then
        begin
            DriverCB.Left    := CFieldPos;
            DriverCB.Top     := VPos;
            DriverCB.Width   := CFieldWidth;
            DriverCB.Parent  := Self;
            DriverCB.Visible := True;

            for DriverNo := 0 to (DriverList.Count-1) do
            begin
                { ensure the driver names are registered. }
                GetOdbDriverClass(DriverList[DriverNo]);

                DriverCB.Items.Add(DriverList[DriverNo]);
            end;

            if (DefaultDriver = '') then
            begin
                ItemIndex := DriverCB.Items.IndexOf(LastDriver);
                if ItemIndex = -1 then
                    ItemIndex := 0;
                DriverCB.ItemIndex := ItemIndex;
            end;

            with TLabel.Create(Self) do
            begin
                Left    := CLabelPos;
                Top     := VPos + DriverCB.Height - Height;
                Caption := 'Driver :';
                Parent  := Self;
            end;

            Inc(VPos, CFieldHeight);
        end;

        if ShowDatabase in Options then
        begin
            DatabaseCB.Left    := CFieldPos;
            DatabaseCB.Top     := VPos;
            DatabaseCB.Width   := CFieldWidth;
            DatabaseCB.Parent  := Self;
            DatabaseCB.Visible := True;

            if DatabaseHistory.GotoFirst then
            repeat
                StringCombo := DatabaseHistory.CurrentObj as TStringCombo;
                DatabaseCB.Items.Add(StringCombo.Str);
            until not DatabaseHistory.GotoNext;

            if (DefaultDatabase = '') and (DatabaseHistory.Size > 0) then
                DatabaseCB.ItemIndex := 0;

            with TLabel.Create(Self) do
            begin
                Left    := CLabelPos;
                Top     := VPos + DatabaseCB.Height - Height;
                Caption := 'Database :';
                Parent  := Self;
            end;

            Inc(VPos, CFieldHeight);
        end;

        if ShowServer in Options then
        begin
            ServerCB.Left    := CFieldPos;
            ServerCB.Top     := VPos;
            ServerCB.Width   := CFieldWidth;
            ServerCB.Parent  := Self;
            ServerCB.Visible := True;

            if ServerHistory.GotoFirst then
            repeat
                StringCombo := ServerHistory.CurrentObj as TStringCombo;
                ServerCB.Items.Add(StringCombo.Str);
            until not ServerHistory.GotoNext;

            if (DefaultServer = '') and (ServerHistory.Size > 0) then
                ServerCB.ItemIndex := 0;

            with TLabel.Create(Self) do
            begin
                Left    := CLabelPos;
                Top     := VPos + ServerCB.Height - Height;
                Caption := 'Server :';
                Parent  := Self;
            end;

            Inc(VPos, CFieldHeight);
        end;

        if ShowUser in Options then
        begin
            UserCB.Left    := CFieldPos;
            UserCB.Top     := VPos;
            UserCB.Width   := CFieldWidth;
            UserCB.Parent  := Self;
            UserCB.Visible := True;

            if UserHistory.GotoFirst then
            repeat
                StringCombo := UserHistory.CurrentObj as TStringCombo;
                UserCB.Items.Add(StringCombo.Str);
            until not UserHistory.GotoNext;

            if (DefaultUser = '') and (UserHistory.Size > 0) then
                UserCB.ItemIndex := 0;

            with TLabel.Create(Self) do
            begin
                Left    := CLabelPos;
                Top     := VPos + UserCB.Height - Height;
                Caption := 'User :';
                Parent  := Self;
            end;
            Inc(VPos, CFieldHeight);

            PasswordEdit.Left    := CFieldPos;
            PasswordEdit.Top     := VPos;
            PasswordEdit.Width   := CFieldWidth;
            PasswordEdit.Parent  := Self;
            PasswordEdit.Visible := True;
            PasswordEdit.PasswordChar := '*';

            with TLabel.Create(Self) do
            begin
                Left    := CLabelPos;
                Top     := VPos + PasswordEdit.Height - Height;
                Caption := 'Password :';
                Parent  := Self;
            end;

            Inc(VPos, CFieldHeight);
        end;
    end;

    Inc(VPos, 6);
    with TBitBtn.Create(Self) do
    begin
        Parent := Self;
        Kind   := bkOk;
        Left   := 120;
        Top    := VPos;
        Width  := 115;
    end;

    with TBitBtn.Create(Self) do
    begin
        Parent := Self;
        Kind   := bkCancel;
        Left   := 248;
        Top    := VPos;
        Width  := 115;
    end;

    Self.Height := VPos + 70;

end;

{--------------------------------------------------------------------------}

constructor TOpenDatabaseDialog.Create( AOwner : TComponent );
begin
    inherited Create(AOwner);

    Options       := [ShowDriver, ShowDatabase];

    FDriverList   := TStringList.Create;
    FHistorySize  := 8;
end;

{--------------------------------------------------------------------------}

destructor TOpenDatabaseDialog.Destroy;
var
    ObjectStream : TObjectStream;
begin
    if not (csDesigning in ComponentState) then
    begin
        if FFullHistoryFile <> '' then
        begin
            ObjectStream := TObjectStream.CreateFromFile( FFullHistoryFile,
                                                          fmCreate );
            try
                ObjectStream.WriteObject(TStringCombo.Create(FLastDriver,nil));
                ObjectStream.WriteObject(FDatabaseHistory);
                ObjectStream.WriteObject(FServerHistory);
                ObjectStream.WriteObject(FUserHistory);
            finally
                ObjectStream.Free;
            end;
        end;
    end;

    FDatabaseHistory.Free;
    FServerHistory.Free;
    FUserHistory.Free;

    FDriverList.Free;

    inherited Destroy;
end;

{--------------------------------------------------------------------------}

procedure TOpenDatabaseDialog.Loaded;
var
    ObjectStream : TObjectStream;
begin
    inherited Loaded;

    if FileExists(FHistoryFile) then
    begin
        { Retrieve the complete history file name in case the
          directory changes during the execution of the program. }
        FFullHistoryFile := ExpandFileName(FHistoryFile);

        ObjectStream := TObjectStream.CreateFromFile( FFullHistoryFile,
                                                      fmOpenRead );
        try
            FLastDriver      := (ObjectStream.ReadObject as TStringCombo).Str;
            FDatabaseHistory := ObjectStream.ReadObject as TUnOrderedList;
            FServerHistory   := ObjectStream.ReadObject as TUnOrderedList;
            FUserHistory     := ObjectStream.ReadObject as TUnOrderedList;
        finally
            ObjectStream.Free;
        end;
    end
    else
    begin
        { assign FFullHistoryFile for the history file to be created
          in the destructor. }
        FFullHistoryFile := FHistoryFile;

        FLastDriver      := '';
        FDatabaseHistory := TUnOrderedList.Create(TStringCombo, CompareStringComboNoCase);
        FServerHistory   := TUnOrderedList.Create(TStringCombo, CompareStringComboNoCase);
        FUserHistory     := TUnOrderedList.Create(TStringCombo, CompareStringComboNoCase);
    end;
end;

{--------------------------------------------------------------------------}

procedure TOpenDatabaseDialog.SetDriverList( const InDriverList : TStrings );
begin
    FDriverList.Assign(InDriverList);
end;

{--------------------------------------------------------------------------}

function TOpenDatabaseDialog.Execute : TObjectDatabase;
var
    LoginDialog  : TLoginDialog;
    StringCombo  : TStringCombo;
begin
    Result := nil;

    LoginDialog := TLoginDialog.CreateStatic(Self);
    try
        if LoginDialog.ShowModal <> mrOk then
            Exit;

        Result := TObjectDatabase.Create(
                       GetOdbDriverClass(LoginDialog.DriverCB.Text),
                       LoginDialog.DatabaseCB.Text );

        Result.Open( LoginDialog.ServerCB.Text,
                     LoginDialog.UserCB.Text,
                     LoginDialog.PasswordEdit.Text);

        FLastDriver := LoginDialog.DriverCB.Text;

        StringCombo := TStringCombo.Create(LoginDialog.DatabaseCB.Text,nil);
        try
            if DatabaseHistory.Search(StringCombo, 1) then
                DatabaseHistory.InsertAtHead(DatabaseHistory.DetachCurrent)
            else
                DatabaseHistory.InsertAtHead(CloneObj(StringCombo));
        finally
            StringCombo.Free;
        end;
        while DatabaseHistory.Size > HistorySize do
        begin
            DatabaseHistory.GotoLast;
            DatabaseHistory.DeleteCurrent;
        end;

        StringCombo := TStringCombo.Create(LoginDialog.ServerCB.Text,nil);
        try
            if ServerHistory.Search(StringCombo, 1) then
                ServerHistory.InsertAtHead(ServerHistory.DetachCurrent)
            else
                ServerHistory.InsertAtHead(CloneObj(StringCombo));
        finally
            StringCombo.Free;
        end;
        while ServerHistory.Size > HistorySize do
        begin
            ServerHistory.GotoLast;
            ServerHistory.DeleteCurrent;
        end;

        StringCombo := TStringCombo.Create(LoginDialog.UserCB.Text,nil);
        try
            if UserHistory.Search(StringCombo, 1) then
                UserHistory.InsertAtHead(UserHistory.DetachCurrent)
            else
                UserHistory.InsertAtHead(CloneObj(StringCombo));
        finally
            StringCombo.Free;
        end;
        while UserHistory.Size > HistorySize do
        begin
            UserHistory.GotoLast;
            UserHistory.DeleteCurrent;
        end;

    finally
        LoginDialog.Free;
    end;
end;

{--------------------------------------------------------------------------}

end.
